/***************************************************************************
                          mem_core.c  -  description
                             -------------------
    begin                : Tue Feb 5 2002
    copyright            : (C) 2002 by Margus Laak
    email                : margus@mail.ee
 ***************************************************************************/

/***************************************************************************
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA *
 ***************************************************************************/

/*
    Functions that define memory allocations, resizings and freengs
*/

#include "mem_core.h"

#ifdef USE_LIBMM

/*
        these function are merely wrappers for libmm ... currently.
        the idea is, that somwhere in the future we might provide
        support for other shared memory allocation tools ... etc etc...
*/

/*
    init memory ...
*/
void *sh_mem_init ( size_t size, char *tmp_file )
{
        return mm_create(size, tmp_file);
}

/*
    destroy EVERYTHING we have
*/
void sh_mem_dispose ( void *p_segm )
{
        MM *sh_mem_segment = (MM *) p_segm;
        
        mm_destroy(sh_mem_segment);
}

void *sh_mem_malloc ( void *p_segm, size_t size )
{
        MM *sh_mem_segment = (MM *) p_segm;
        return mm_malloc(sh_mem_segment, size);
}

void *sh_mem_realloc ( void *p_segm, void *ptr, size_t size )
{
        MM *sh_mem_segment = (MM *) p_segm;
        
        return mm_realloc(sh_mem_segment, ptr, size);
}

void sh_mem_free ( void *p_segm, void *ptr )
{
        MM *sh_mem_segment = (MM *) p_segm;
        mm_free(sh_mem_segment, ptr);
}

char *sh_mem_strdup ( void *p_segm, const char *str )
{
        MM *sh_mem_segment = (MM *) p_segm;

        CHECK_NULL(str)

        return mm_strdup(sh_mem_segment, str);
}

int sh_mem_lock ( void *p_segm, mm_lock_mode mode )
{
        MM *sh_mem_segment = (MM *) p_segm;
        
        return mm_lock(sh_mem_segment, mode);

}

int sh_mem_unlock ( void *p_segm )
{
        MM *sh_mem_segment = (MM *) p_segm;
        
        return mm_unlock(sh_mem_segment);
}

size_t sh_mem_available ( void *p_segm )
{
        MM *sh_mem_segment = (MM *) p_segm;
        
        return mm_available(sh_mem_segment);
}

int sh_mem_detach ( void * p_segm )
{
        MM * sh_mem_segment = (MM *) p_segm;

        return mm_detach(sh_mem_segment);
}

int sh_mem_shmid ( void * p_segm )
{
        MM * sh_mem_segment = (MM *) p_segm;

        return mm_shmid(sh_mem_segment);
}

int sh_mem_mapcount ( void * p_segm )
{
        return mm_mapcount((MM *)p_segm);
}

int sh_mem_addmap ( void * p_segm )
{
        return mm_addmap((MM *)p_segm);
}

int sh_mem_remmap ( void * p_segm )
{
        return mm_remmap((MM *)p_segm);
}

#endif

#ifdef USE_MALLOC
void *sh_mem_init ( size_t size, char *tmp_file )
{
        return (void *)1;
}

/*
    destroy EVERYTHING we have
*/
void sh_mem_dispose ( void *p_segm )
{
        return;
}

void *sh_mem_malloc ( void *p_segm, size_t size )
{
        return malloc(size);
}

void *sh_mem_realloc ( void *p_segm, void *ptr, size_t size )
{
        return realloc(ptr, size);
}

void sh_mem_free ( void *p_segm, void *ptr )
{
        free(ptr);
}

char *sh_mem_strdup ( void *p_segm, const char *str )
{
        int len;
        char * ptr;

        CHECK_NULL(str)

        len = strlen(str);

        ptr = malloc((len+1) * sizeof(char));
        strcpy(ptr, str);

        return ptr;
}

int sh_mem_lock ( void *p_segm, mm_lock_mode mode )
{
        return 1;
}

int sh_mem_unlock ( void *p_segm )
{
        return 1;
}

size_t sh_mem_available ( void *p_segm )
{
        return 1000;
}

int sh_mem_detach ( void * p_segm )
{
        return 1;
}

int sh_mem_shmid ( void * p_segm )
{
        return 1;
}
int sh_mem_mapcount ( void * p_segm )
{
        return 1;
}

int sh_mem_addmap ( void * p_segm )
{
        return 1;
}

int sh_mem_remmap ( void * p_segm )
{
        return 0;
}

#endif
