/***************************************************************************
                          shmap_btree.h  -  description
                             -------------------
    begin                : Sat May 11 2002
    copyright            : (C) 2002 by Margus Laak
    email                : margus@mail.ee
 ***************************************************************************/

/***************************************************************************
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA *
 ***************************************************************************/

#include "shmap.h"
#include "shmap_internals.h"

#define SHMAP_BTREE_MODULE "Shmap binary tree tree module"
#define SHMAP_BTREE_VERSION "0.1"

typedef struct _shmap_btree _shmap_btree;
struct _shmap_btree
{
        _shmap_btree   * parent;  // upper level
        _shmap_btree   * left;    // equal or less
        _shmap_btree   * right;   // more
        _shmap_value * value;
};

// for the toplevel btree tree
typedef struct _shmap_btree_top _shmap_btree_top;
struct _shmap_btree_top
{
        _shmap_btree   * tree;
        int            left_items;
        int            right_items;
};

void          * shmap_btree_init ( void * mem_segm, void * p_opts );
_shmap_value  * shmap_btree_get ( shmap * map, const char * key, int cached );
int             shmap_btree_add ( shmap * map, const char * key, void * value, int opts, shmap_cleanup cleanup );
int             shmap_btree_del ( shmap * map, const char * key, shmap_cleanup cleanup );
_shmap_value  * shmap_btree_get_sub ( shmap * map, const char * key, int keylen, int pos, int cached );
int             shmap_btree_del_sub ( shmap * map, const char * key, int keylen, shmap_cleanup cleanup );
int             shmap_btree_clear ( shmap * map, shmap_cleanup cleanup );
int             shmap_btree_dispose ( shmap * map, shmap_cleanup cleanup );


_shmap_btree  * _shmap_btree_get ( _shmap_btree_top * toplevel, const char * key, int keylen, int keyhash, int pos );
int             _shmap_btree_add ( void * p_mem, _shmap_btree_top * toplevel, _shmap_value * value, shmap_cleanup cleanup );
int             _shmap_btree_remove ( void * p_mem, _shmap_btree_top * top, _shmap_btree * btree, shmap_cleanup cleanup );
int             _shmap_btree_clear ( void * p_mem, _shmap_btree_top * toplevel, shmap_cleanup cleanup );
_shmap_btree  * _shmap_btree_item_init ( void * p_mem, _shmap_value * value );
int             _shmap_btree_item_place ( void * p_mem, _shmap_btree_top * top, _shmap_btree * item, shmap_cleanup cleanup );
int             _shmap_btree_item_delete ( void * p_mem, _shmap_btree * item, shmap_cleanup cleanup );
int             _shmap_btree_reset_balance ( void * p_mem, _shmap_btree_top * top );

