/***************************************************************************
                          shmap_hm.c  -  description
                             -------------------
    begin                : Tue Feb 5 2002
    copyright            : (C) 2002 by Margus Laak
    email                : margus@mail.ee
 ***************************************************************************/

/***************************************************************************
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA *
 ***************************************************************************/


#include "shmap_hm.h"

/*
        SHMAP_TYPE_HM implementation

        the reason why some functions are only aliases for another ones
          is that in the future probably more things will be added that require
          this kind of approach


        TODO:
          * caching system needs implementation
          * data dumping function needs implementation
*/


struct _shmap_hm_entry
{
        _shmap_value** keys;

        int max_keys;
        int current_keys;

        _shmap_hm_entry * nextlevel;
        _shmap_hm_entry * prevlevel;

};

shmap_module hm_module =
        {
        SHMAP_HM_MODULE,
        SHMAP_HM_VERSION,
        SHMAP_TYPE_HM,
        shmap_hm_init,
        shmap_hm_get_value,
        shmap_hm_add_value,
        shmap_hm_del_value,
        shmap_hm_get_sub,
        shmap_hm_del_sub,
        shmap_hm_clear,
        shmap_hm_dispose,
        NULL, // cache init
        NULL, // cache clean
        NULL, // cache dispose
        shmap_hm_dump
        };

/*
	shmap_init
*/
void * shmap_hm_init ( void * p_mem, void * opts )
{
        return _shmap_hm_core_newlevel(p_mem, NULL);
}

_shmap_value * shmap_hm_get_value ( shmap * map, const char * key, int cached )
{
        return _shmap_hm_core_get_value(map->toplevel, key);
}

int shmap_hm_add_value ( shmap * map, const char * key, void * value, int opts, shmap_cleanup cleanup )
{
        return _shmap_hm_core_add_value(map->mem_segm, map->toplevel, key, value, opts, cleanup);
}

int shmap_hm_del_value ( shmap * map, const char * key, shmap_cleanup cleanup )
{
        return _shmap_hm_core_del_value(map->mem_segm, &map->toplevel, key, cleanup);
}

_shmap_value * shmap_hm_get_sub ( shmap * map, const char * key, int keylen, int pos, int cached )
{
        return _shmap_hm_core_get_sub(map->toplevel, key, keylen, pos);
}

int shmap_hm_del_sub ( shmap * map, const char * key, int keylen, shmap_cleanup cleanup )
{
        int del_items = 0;
        _shmap_value * val;

        while ( (val = _shmap_hm_core_get_sub(map->toplevel, key, keylen, 0)) != NULL )
        {
                if ( _shmap_hm_core_del_value(map->mem_segm, &map->toplevel, val->key, cleanup) )
                {
                        del_items++;
                }
        }

        return del_items;
}

int shmap_hm_clear ( shmap * map, shmap_cleanup cleanup )
{
        return _shmap_hm_core_del_values(map->mem_segm, &map->toplevel, cleanup );
}

int shmap_hm_dispose ( shmap * map, shmap_cleanup cleanup )
{
        if ( _shmap_hm_core_del_values(map->mem_segm, &map->toplevel, cleanup) )
        {
                return 1;
        }
        else
                return 0;
}

void shmap_hm_dump ( shmap * map, int with_values )
{
// needs implementation
}

/*********************************************

        core functions start here

**********************************************/


_shmap_value * _shmap_hm_core_get_value ( void * p_toplevel, const char * key )
{
        int keyhash, marker, i;
        _shmap_hm_entry * entries = (_shmap_hm_entry *) p_toplevel;
        
        keyhash = _core_get_hashvalue(key);
        
        while ( entries != NULL )
        {
                for ( i = 0 ; i < entries->max_keys ; i++ )
                {
                        marker = (keyhash+i)%entries->max_keys;
                        if (  entries->keys[marker] != NULL && 
                               entries->keys[marker]->keyhash == keyhash &&
                                strcmp(entries->keys[marker]->key, key) == 0 )
                        {
                                return entries->keys[marker];
                        }
                }
                
                entries = entries->nextlevel;
        }
        
        return NULL;

}

int _shmap_hm_core_add_value ( void * p_mem, void * p_toplevel, const char * key, void * value, int opts, shmap_cleanup cleanup )
{
        int keyhash, marker, i, status = 2;
        _shmap_hm_entry * entries = (_shmap_hm_entry *) p_toplevel;

        keyhash = _core_get_hashvalue(key);

        marker = 0;
        while ( entries != NULL )
        {
                for ( i = 0 ; i < entries->max_keys ; i++ )
                {
                        marker = (keyhash+i)%entries->max_keys;
                        if ( entries->keys[marker] == NULL || (entries->keys[marker]->keyhash == keyhash && strcmp(key, entries->keys[marker]->key) == 0 ) )
                        {
                                if ( entries->keys[marker] )
                                {
                                        status = 1;
                                        shmap_value_dispose(p_mem, entries->keys[marker], cleanup);
                                }

                                entries->keys[marker] = shmap_value_init(p_mem, (void *)key, keyhash, (void *)value, opts);
                                // check if the returned value isn't NULL
                                if ( entries->keys[marker] == NULL )
                                        return 0;
                                entries->keys[marker]->keyhash = keyhash;

                                return status;
                        }
                }
                
                if ( entries->nextlevel == NULL )
                        entries->nextlevel = _shmap_hm_core_newlevel (p_mem, entries);

                entries = entries->nextlevel;
        }
        
        return 0;
}

int _shmap_hm_core_del_value ( void * p_mem, void ** p_toplevel, const char * key, shmap_cleanup cleanup )
{
        int keyhash, marker, i;
        _shmap_value * entry;
        _shmap_hm_entry  * entries = (_shmap_hm_entry *) *p_toplevel;
        
        keyhash = _core_get_hashvalue(key);

        while ( entries != NULL )
        {
                for ( i = 0 ; i < entries->max_keys ; i++ )
                {
                        marker = (keyhash + i) % entries->max_keys;
                        if (   entries->keys[marker] != NULL &&
                                entries->keys[marker]->keyhash == keyhash &&
                                 strcmp(entries->keys[marker]->key, key) == 0 )
                        {
                                entry = entries->keys[marker];
                                shmap_value_dispose(p_mem, entry, cleanup);
                                entries->keys[marker] = NULL;
                                entries->current_keys--;
                                
                                if ( entries->current_keys == 0 )
                                {
                                        _shmap_hm_core_del_entrylevel (p_toplevel, entries, 0, cleanup);
                                }
                        }
                }
                entries = entries->nextlevel;
        }
        
        return 1;
}

_shmap_value * _shmap_hm_core_get_sub ( void * p_toplevel, const char * key, int keylen, int pos )
{
        int count = 0, i;
        _shmap_hm_entry * entries = (_shmap_hm_entry *) p_toplevel;

        while ( entries != NULL )
        {
                for ( i = 0 ; i < entries->max_keys ; i++ )
                {
                        if ( entries->keys[i] != NULL && strncmp(entries->keys[i]->key, key, keylen) == 0 )
                        {
                                if ( pos == count )
                                        return entries->keys[i];
                                else
                                        count++;
                        }
                }
                entries = entries->nextlevel;
        }
        
        return NULL;
}

/*
        deletes all values from map and reconnects the toplevel
*/

int _shmap_hm_core_del_values ( void * p_mem, void ** p_toplevel, shmap_cleanup cleanup )
{
        _shmap_hm_entry * entries = (_shmap_hm_entry *) *p_toplevel;

        while ( entries != NULL )
        {
                // also reconnect the toplevel, depending on how we deleted elements
                if ( _shmap_hm_core_del_entrylevel ( p_mem, entries, 0, cleanup ) == 2 )
                {
                        *p_toplevel = entries->nextlevel;
                }
                else
                {
                        *p_toplevel = entries;
                }
                entries = entries->nextlevel;
        }

        return 1;
}

/*
        creates a new empty level and connects it to previous level
*/
_shmap_hm_entry * _shmap_hm_core_newlevel ( void * p_mem, _shmap_hm_entry * prevlevel )
{
        _shmap_hm_entry * entry;

        entry = sh_mem_malloc(p_mem, sizeof(_shmap_hm_entry));
        
        entry->keys = (_shmap_value **) sh_mem_malloc(p_mem, sizeof(_shmap_value *) * SHMAP_HM_VALUES);
        memset(entry->keys, 0, sizeof(_shmap_value *) * SHMAP_HM_VALUES);
        
        entry->prevlevel    = prevlevel;
        entry->nextlevel    = NULL;
        entry->current_keys = 0;
        entry->max_keys     = SHMAP_HM_VALUES;
        
        return entry;
}

/*
        deletes an entry level, only if specified level is not the only one left
        in any case all values will be deleted
        
        use force = 1 to delete even, when one level is left

        return:
                1 - only elements where cleaned out
                2 - if also the entrylevel is freed
*/
int _shmap_hm_core_del_entrylevel ( void * p_mem, _shmap_hm_entry * entrylevel, int force, shmap_cleanup cleanup )
{
        int i;
        
        for ( i = 0 ; i < entrylevel->max_keys ; i++ )
        {
                if ( entrylevel->keys[i] )
                {
                        shmap_value_dispose(p_mem, entrylevel->keys[i], cleanup);
                        entrylevel->keys[i] = NULL;
                }
        }
        
        // make sure we don't delete wrong level
        if ( force || entrylevel->nextlevel || entrylevel->prevlevel )
        {
                // first reconnect missing links
                if ( entrylevel->prevlevel )
                {
                        entrylevel->prevlevel->nextlevel = entrylevel->nextlevel;
                }
                if ( entrylevel->nextlevel )
                {
                        entrylevel->nextlevel->prevlevel = entrylevel->prevlevel;
                }


                sh_mem_free(p_mem, entrylevel->keys);
                sh_mem_free(p_mem, entrylevel);
                return 2;
        }
        
        return 1;
}

