/***************************************************************************
                          shmap_internals.h  -  description
                             -------------------
    begin                : Sun Apr 28 2002
    copyright            : (C) 2002 by Margus Laak
    email                : margus@mail.ee
 ***************************************************************************/

/***************************************************************************
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA *
 ***************************************************************************/

#ifndef _SHMAP_INTERNALS_I
#define _SHMAP_INTERNALS_I
#include <mm.h>

//  [key][value][][][][][][][] ...
#define __SHMAP_FREE_KEY   0x1
#define __SHMAP_FREE_VALUE 0x2

typedef struct _shpool _shpool;
typedef struct _shpool_list _shpool_list;

struct _shmap_value
{
        char * key;

        void * value;
        char   options;

        int keyhash;
};

_shmap_value * shmap_value_init ( void * p_mem, void * key, int keyhash, void * value, int opts );
int            shmap_value_dispose ( void * p_mem, _shmap_value * value, shmap_cleanup );
int            shmap_value_replace_value ( void * p_mem, _shmap_value * p_val, void * val, shmap_cleanup );
int            shmap_value_replace ( void * p_mem, _shmap_value * p_val, _shmap_value * value, shmap_cleanup cleanup );
_shmap_value * shmap_pool_value_init ( _shpool * pool, void * key, int keyhash, void * value, int opts );
int            shmap_pool_value_dispose ( _shpool * pool, _shmap_value * value, shmap_cleanup cleanup );
int            shmap_pool_replace_value ( _shpool * pool, _shmap_value * p_val, void * val, shmap_cleanup cleanup );
int            shmap_pool_replace ( _shpool * pool, _shmap_value * p_val, _shmap_value * value, shmap_cleanup cleanup );

struct shmap_module {
        const char *   module_name;
        const char *   module_version;
        int            module_type;
        void *        (*shmap_init)(void *, void *);
        _shmap_value *(*shmap_get_value)(shmap *, const char *, int);
        int           (*shmap_add_value)(shmap *, const char *, void *, int, shmap_cleanup ); // map, key, value pointer, value options
        int           (*shmap_del_value)(shmap *, const char *, shmap_cleanup );
        _shmap_value *(*shmap_get_sub)(shmap *, const char *, int, int, int); // map, key, keylen, pos, cached
        int           (*shmap_del_sub)(shmap*, const char *, int, shmap_cleanup );
        int           (*shmap_clear)(shmap *, shmap_cleanup );
        int           (*shmap_dispose)(shmap *, shmap_cleanup );
        void *        (*shmap_cache_init)(void *);
        void          (*shmap_cache_clean)(void *);
        int           (*shmap_cache_dispose)(void *, void **);
        void          (*shmap_dump)(shmap *, int);
        };


unsigned int _core_get_hashvalue ( const char * );

shmap_lock * _core_lock_init ( void *, const char * name );
void         _core_lock_dispose ( shmap_lock *, void * );
void         _core_lock ( shmap_lock *, shmap_lock_mode );
void         _core_unlock ( shmap_lock * );
int          _core_lock_attach ( shmap_lock * lock );

shmap_perm * _core_get_perm ( shmap * map );
int          _core_set_perm ( shmap * map, shmap_perm * perm );

#define CORE_LOCK(l,m)          if ( l != NULL ) _core_lock(l,SHMAP_LOCK_WRITE);
#define CORE_UNLOCK(l)          if ( l != NULL ) _core_unlock(l);

union semun {
        int val;                    /* value for SETVAL */
        struct semid_ds *buf;       /* buffer for IPC_STAT, IPC_SET */
        unsigned short int *array;  /* array for GETALL, SETALL */
        struct seminfo *__buf;
};

/**
 *  Module pointer definitions
 */
extern shmap_module ll_module;
extern shmap_module hm_module;
extern shmap_module btree_module;
extern shmap_module fifo_module;

/*
        memory stuff
 */
//#define USE_LIBMM
#define USE_MALLOC

void       * sh_mem_init ( size_t, char * );
void         sh_mem_dispose ( void * );
void       * sh_mem_malloc ( void *, size_t );
void       * sh_mem_realloc ( void *, void *, size_t );
void         sh_mem_free ( void *, void * );
int          sh_mem_lock ( void *, mm_lock_mode );
int          sh_mem_unlock ( void * );
char       * sh_mem_strdup ( void *, const char * );
size_t       sh_mem_available ( void * );
int          sh_mem_detach ( void * p_segm );
int          sh_mem_shmid ( void * p_segm );
int          sh_mem_mapcount ( void * p_segm );
int          sh_mem_addmap ( void * p_segm );
int          sh_mem_remmap ( void * p_segm );
/*
        pooling functions
*/
_shpool * sh_pool_init ( void * p_mem );
void      sh_pool_freeall ( _shpool * pool );
void      sh_pool_dispose ( _shpool * pool );
void    * sh_pool_malloc ( _shpool *pool, size_t size );
void    * sh_pool_realloc ( _shpool *pool, void *ptr, size_t size );
void      sh_pool_free ( _shpool * pool, void *ptr );
char    * sh_pool_strdup ( _shpool * pool, const char *str );
int       sh_pool_lock ( _shpool * pool, mm_lock_mode mode );
int       sh_pool_unlock ( _shpool * pool );
size_t    sh_pool_available ( _shpool * pool );
int       sh_pool_detach ( _shpool * pool );
int       sh_pool_shmid ( _shpool * pool );

// pooling support
#define POOL_LEVEL_SIZE 128

struct _shpool
{
        void         * memsegm; // the memory segment of pool
        _shpool_list * top;     // the toplevel item of list
        _shpool_list * current; // the current level where we add items
};

struct _shpool_list
{
        void    * ptrlist[POOL_LEVEL_SIZE];
        int       listsize;   // the count of maximum items in this level
        int       cursor;     // the cursor position
        int       removed;    // the count of removed items on this level

        _shpool_list * next;  // pointer to next level of items
};

#define DEBUG(t)        printf(t); fflush(stdout);
/*
        some utilities for general usage
*/
#define CHECK_NULL(m) if ( m == NULL ) return NULL;
#define CHECK_NULL0(m) if ( m == NULL ) return 0;
#define CHECK_NULL_(m) if ( m == NULL ) return;



#endif