/***************************************************************************
                          shmap_value.c  -  description
                             -------------------
    begin                : Wed Feb 27 2002
    copyright            : (C) 2002 by Margus Laak
    email                : margus@mail.ee
 ***************************************************************************/

/***************************************************************************
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA *
 ***************************************************************************/

#include "shmap_value.h"


_shmap_value * shmap_value_init ( void * p_mem, void * key, int keyhash, void * value, int opts )
{
        _shmap_value * ret_value;

        CHECK_NULL(p_mem)

        ret_value = (_shmap_value *) sh_mem_malloc(p_mem, sizeof(_shmap_value));
        if ( ret_value != NULL )
        {
                ret_value->key     = key;
                ret_value->value   = value;
                ret_value->options = opts;
                ret_value->keyhash = keyhash;
        }

        return ret_value;
}

_shmap_value * shmap_pool_value_init ( _shpool * pool, void * key, int keyhash, void * value, int opts )
{
        _shmap_value * ret_value;

        CHECK_NULL(pool)

        ret_value = (_shmap_value *) sh_pool_malloc(pool, sizeof(_shmap_value));
        if ( ret_value != NULL )
        {
                ret_value->key     = key;
                ret_value->value   = value;
                ret_value->options = opts;
                ret_value->keyhash = keyhash;
        }

        return ret_value;
}

/*
 * Cleans the map item value
 *
 */
int shmap_value_dispose ( void * p_mem, _shmap_value * value, shmap_cleanup cleanup )
{
        int opts;

        if ( cleanup != NULL )
        {
                cleanup(value, &opts);
        }
        else
        {
                opts = value->options;
        }

        if ( opts & __SHMAP_FREE_KEY )
                sh_mem_free(p_mem, value->key);
        if ( opts & __SHMAP_FREE_VALUE )
                sh_mem_free(p_mem, value->value);
        sh_mem_free(p_mem, value);

        return 1;
}

int shmap_pool_value_dispose ( _shpool * pool, _shmap_value * value, shmap_cleanup cleanup )
{
        int opts;

        if ( cleanup != NULL )
        {
                cleanup(value, &opts);
        }
        else
        {
                opts = value->options;
        }

        if ( opts & __SHMAP_FREE_KEY )
                sh_pool_free(pool, value->key);
        if ( opts & __SHMAP_FREE_VALUE )
                sh_pool_free(pool, value->value);
        sh_pool_free(pool, value);

        return 1;
}

/*
        only rewrites the value pointer for another one
        if necessary also free-s the old pointer
*/
int shmap_value_replace_value ( void * p_mem, _shmap_value * p_val, void * val, shmap_cleanup cleanup )
{
        int opts;

        if ( cleanup != NULL )
        {
                cleanup(p_val, &opts);
        }
        else
        {
                opts = p_val->options;
        }

        if ( opts & __SHMAP_FREE_VALUE )
                sh_mem_free(p_mem, p_val->value);
        p_val->value = val;

        return 1;
}

int shmap_pool_replace_value ( _shpool * pool, _shmap_value * p_val, void * val, shmap_cleanup cleanup )
{
        int opts;

        if ( cleanup != NULL )
        {
                cleanup(p_val, &opts);
        }
        else
        {
                opts = p_val->options;
        }

        if ( opts & __SHMAP_FREE_VALUE )
                sh_pool_free(pool, p_val->value);
        p_val->value = val;

        return 1;
}

int shmap_value_replace ( void * p_mem, _shmap_value * p_val, _shmap_value * value, shmap_cleanup cleanup )
{
        int opts;

        if ( cleanup != NULL )
        {
                cleanup(p_val, &opts);
        }
        else
        {
                opts = p_val->options;
        }

        if ( opts & __SHMAP_FREE_VALUE )
                sh_mem_free(p_mem, p_val->value);
        p_val->value = value->value;
        if ( opts & __SHMAP_FREE_KEY )
                sh_mem_free(p_mem, p_val->key);
        p_val->key = value->key;

        p_val->keyhash = value->keyhash;

        return 1;
}

int shmap_pool_replace ( _shpool * pool, _shmap_value * p_val, _shmap_value * value, shmap_cleanup cleanup )
{
        int opts;

        if ( cleanup != NULL )
        {
                cleanup(p_val, &opts);
        }
        else
        {
                opts = p_val->options;
        }

        if ( opts & __SHMAP_FREE_VALUE )
                sh_pool_free(pool, p_val->value);
        p_val->value = value->value;
        if ( opts & __SHMAP_FREE_KEY )
                sh_pool_free(pool, p_val->key);
        p_val->key = value->key;

        p_val->keyhash = value->keyhash;

        return 1;
}

